@@ -1,22 +1,18 @@
-# ## P2J - Python 2 Jupyter utility script
-# ## TODO LIST
+# ## J2Py - Python <=> Jupyter utility script
+# run this script to convert easily and reversably scripts and notebooks, while keeping the formatted cells, comments and outputs
 # ### j2py
-# - add options
 # - build shebangs from python version info
-# - improve matches
 # ### GUI
 # -   add icon
-# -   add options
 # -   add confirmation button if a file is going to be overwritten
 
-# run this script to convert easily and reversably scripts and notebooks, while keeping the formatted cells, comments and outputs
-# In[1]:
 import sys
 import re
 import random as rnd
 import json
 from copy import deepcopy
 from pathlib import Path
+from hashlib import md5
 
 TITLE = r"""
    ___  ______      
@@ -29,6 +25,15 @@ TITLE = r"""
               |___/ 
 """
 
+__all__ = [
+    'TITLE',
+    'OPTIONS',
+    'py2j',
+    'j2py',
+    'guimode',
+    'main',
+]
+
 
 class reg():
     """Store here all the regexes used in the script"""
@@ -124,10 +129,8 @@ def fixrow(s):
     if s[-1] != '\n':
         return s + '\n'
     return s
-    # return re.sub('\n*$', '\n', s) # not working always
 
 
-# In[3]:
 def py2j(dir_input: str, dir_output: str) -> str:
     p(dir_input, '->', dir_output)
     with open(dir_input) as py:
@@ -160,25 +163,17 @@ def py2j(dir_input: str, dir_output: str) -> str:
                     # TODO duplicate match definition. not soo good
                     rowmatch = re.match(reg.scriptrow, temprow).groups()
                     if rowmatch[0] == '':
-                        # all the past rows are just useless blank
+                        # all the past rows are just blank and not part of a func/class
                         # a new cell is created here
                         if cur['source']:
                             split_cells.append(cur)
                         cur = deepcopy(script_cell_template)
                         cur['id'] = gen_id()
                         cur['cell_type'] = 'code'
-                        # p(f'BLANK ROWS DELETION!')
-                    else:
-                        # this row and the previous ones are part of a function
-                        # better not break them
-                        ...
-                        # p(f'BLANK ROWS SAVED! {rowmatch[1:]}')
-
                     cur['source'].extend(temp)
                     break
             row = temprow
-            # p(temprow)
-
+            # p(temprow
         cell_sep = re.match(reg.code_separator, row)
         mkdn_sep = re.match(reg.markdown, row)
         # when a cell separator is hit, split_cells gets updated and a new cell is generated
@@ -217,9 +212,7 @@ def py2j(dir_input: str, dir_output: str) -> str:
         **notebook_info
     }
 
-    with open(dir_output, 'w' if OPTIONS['overwrite'] else 'x') as out_file:
-        out_file.write(json.dumps(json_output, indent=True))
-        p(f"saved to file {dir_output}")
+    return savefile(dir_output, json.dumps(json_output, indent=True))
 
 
 def j2py(dir_input: str, dir_output: str) -> str:
@@ -243,8 +236,7 @@ def j2py(dir_input: str, dir_output: str) -> str:
         else:
             raise BaseException(f'Unrecognized cell type: {cell["cell_type"]}')
 
-    with open(dir_output, 'w' if OPTIONS['overwrite'] else 'x') as py_out:
-        py_out.write(''.join(cell_rows))
+    return savefile(dir_output, ''.join(cell_rows))
 
 
 def runconversion(dir_input: str, dir_output: str) -> str:
@@ -252,49 +244,37 @@ def runconversion(dir_input: str, dir_output: str) -> str:
     p(f"{repr(dir_input)} --> {repr(dir_output)}")
 
     if dir_input.endswith('.py') and dir_output.endswith('.ipynb'):
-        py2j(dir_input, dir_output)
+        out = py2j(dir_input, dir_output)
     elif dir_input.endswith('.ipynb') and dir_output.endswith('.py'):
-        j2py(dir_input, dir_output)
-    elif re.search(r"(\.ipynb$|\.py$)") and dir_output == '':
-        runconversion(dir_input, convert_name(dir_input))
+        out = j2py(dir_input, dir_output)
+    elif re.search(r"(\.ipynb$|\.py$)", dir_input) and dir_output == '':
+        out = runconversion(dir_input, convert_name(dir_input))
     else:
         raise Exception(
             f"Invalid names for conversion: '{dir_input}' and '{dir_output}'")
+    return "File converted" if out else "File not converted"
 
 
-def demo(truename=False):
-    # TODO deprecate
-    py2j('j2py.py', 'j2py_demo.ipynb')
-    j2py('j2py_demo.ipynb', 'j2py_demo.py')
-    for i in range(50):
-        py2j('j2py_demo.py', 'j2py_demo.ipynb')
-        j2py('j2py_demo.ipynb', 'j2py_demo.py')
+def savefile(dir_output, filecontent):
+    """Returns
+    True if the file was saved
+    False in case the file the file exists and overwrite disabled"""
+    opt = 'w' if OPTIONS['overwrite'] else 'x'
+    with open(dir_output, opt) as out_file:
+        out_file.write(filecontent)
+        p(f"saved to file {dir_output}")
+    return True
 
 
-def simplewindowselect():
-    # TODO deprecate
-    import tkinter as tk
-    from tkinter import ttk
-    from tkinter.filedialog import askopenfilename
-    from tkinter.simpledialog import askstring
-    # Store the path to the selected file
-    tk.Tk().withdraw()  # Do not show root window as tk
-    start_folder = Path('.')
-    startfile = askopenfilename(initialdir=start_folder)
+def breakstr(s, cols=45):
+    return '\n'.join(s[i:i+cols] for i in range(0, len(s), cols))
 
-    if startfile == '':
-        p('You did not select any file.')
-        input('Press enter to close...')
-    else:
-        p('Source selected: ' + startfile)
-        outputfile = askstring(
-            "Output file name",
-            prompt=f"Select the name for the output file",
-            initialvalue=convert_name(startfile))
-        p(outputfile)
-        runconversion(startfile, outputfile)
 
-    p("Done")
+def samedata(f0: str, f1: str) -> bool:
+    """Checks the content of two strings, returning true if they are not updated"""
+    h0 = md5(f0.encode()).hexdigest()
+    h1 = md5(f1.encode()).hexdigest()
+    return h0 == h1
 
 
 def guimode():
@@ -333,9 +313,9 @@ def guimode():
                            command=lambda: inner_run())
     runbutton.grid(column=0, row=2, sticky="nw")
     # status information
-    status = tk.Label(inputframe, height=1, width=50,
-                      text="Status Information")
-    status.grid(column=0, row=3, columnspan=2, sticky='nw')
+    status = tk.Label(inputframe, width=50,
+                      text="Status Information", anchor='w')
+    status.grid(column=0, row=3, columnspan=2, sticky='ew')
 
     # option buttons - global, p2j and j2p
     overwrite = tk.BooleanVar(value=OPTIONS['overwrite'])
@@ -397,7 +377,10 @@ def guimode():
                 out = Path(askopenfilename(initialdir='.'))
                 inputvalue.set(out)
                 outputvalue.set(convert_name(str(out)))
-                status.config(text=f"Input set")
+                if re.match(r".*\.py$", str(out)):
+                    status.config(text=f"Py -> J")
+                else:
+                    status.config(text=f"J -> Py")
             else:
                 out = Path(
                     askdirectory(initialdir='.')) / convert_name(
@@ -413,16 +396,16 @@ def guimode():
             fout = outputtext.get()
             output = runconversion(fin, fout)
             status.config(text=output)
+        except FileExistsError:
+            status.config(text=f"File {Path(fout).name} ALREADY EXISTS!")
         except Exception as e:
-            status.config(text=f"Error: {e}")
+            print(f"Error: {e}")
+            status.config(text=breakstr(f"Error: {e}"))
 
     gui.mainloop()
     p("Gui closed")
 
 
-# In[4]:
-
-
 def print_help():
     print(TITLE)
     print("J2Py conversion utility")
@@ -435,25 +418,23 @@ def print_help():
     print("\tPrints this message and exits")
     print(" -m")
     print("\tAllows user to insert manually the input and output file")
-    print("  Jupyter --> Python options:")
+    print("  Python --> Jupyter options:")
     print("--use-blanks")
     print("\tSPLIT code whenever there are blank lines")
     print("--ignore-ins")
     print("\tIGNORE the In[#] separators when")
     print("--no-markdown")
-    print("\tWill NOT try to convert markdown snippets to code")
-    print("  Python --> Jupyter options")
+    print("\tWill ignore double hashtag rows (usually marking Markdown data)")
+    print("  Jupyter --> Python options")
     print("--delete-markdown")
-    print("\tIgnore markdown and raw cells")
+    print("\tIGNORE markdown and raw cells")
     print("--remove-separators")
-    print("\tDoes not paste separators (makes a cleaner output)")
+    print("\tDoes NOT PASTE SEPARATORS (makes a cleaner output)")
     print("Other options")
     print("  --overwrite")
     print("\tWill OVERWRITE the destination file if already in the system")
     print("")
 
-# In[13]:
-
 
 def main():
     if len(sys.argv) == 1:
@@ -467,6 +448,8 @@ def main():
     dargs = {'files': []}
     for a in iargs:
         p(a)
+        # match_ext = re.match(reg.arg_ext, a)
+        # match_short =re.match(reg.arg_short, a)
         if a in ['-h', '--help']:
             print_help()
             return
@@ -494,9 +477,9 @@ def main():
             OPTIONS['p2j']['markdown_separators'] = False
         # j2py options
         elif a in ['--delete-markdown']:
-            OPTIONS['delete_markdown'] = True
+            OPTIONS['j2py']['delete_markdown'] = True
         elif a in ['--remove-separators']:
-            OPTIONS['keep_separators'] = False
+            OPTIONS['j2py']['keep_separators'] = False
         # other options
         elif a in ['--overwrite']:
             OPTIONS['overwrite'] = True
@@ -505,7 +488,8 @@ def main():
 
     p(dargs)
 
-    assert len(dargs['files']) > 0, "You must select at 1 file to convert"
+    assert len(dargs['files']
+               ) > 0, "You must select at least 1 file to convert"
 
     if len(dargs['files']) & 1 == 1:  # evenize the file arguments
         dargs['files'].append(convert_name(dargs[-1]))
